import dbm
import pytest

from pathlib import Path

import rss


@pytest.fixture(scope='function')
def get_raw_mock_avtonom(monkeypatch):
    with open((Path(__file__) / '..' / 'rss' / 'avtonom_full_rss').resolve(), 'r') as f:
        raw = f.read()
    monkeypatch.setattr(rss, '_get_raw', lambda url: raw)


@pytest.fixture(scope='function')
def get_raw_mock_opennet(monkeypatch):
    with open((Path(__file__) / '..' / 'rss' / 'opennews_full_rss').resolve(), 'r') as f:
        raw = f.read()
    monkeypatch.setattr(rss, '_get_raw', lambda url: raw)


@pytest.fixture(scope='function')
def db(tmp_path):
    with dbm.open(str(tmp_path / 'test'), 'n') as impl:
        yield impl


def test_avtonom(get_raw_mock_avtonom, db):
    results = []
    def _processor(content, media):
        results.append((content, media))
        return 'T'
    rss.process_new_items(db, 'https://avtonom.org-pytest/rss_full', _processor)

    assert list(reversed([media for content, media in results])) == [
        'https://avtonom.org/sites/default/files/store/poster2022_ru-780x1103.png',
        'https://avtonom.org/sites/default/files/store/taylor-768x1024.jpeg',
        'https://avtonom.org/sites/default/files/store/20220731_221430_0.jpg',
        'https://avtonom.org/sites/default/files/store/20220731_184703.jpg',
        'https://avtonom.org/sites/default/files/store/starship.png',
        'https://avtonom.org/sites/default/files/store/fiar_rus.jpg',
        'https://avtonom.org/sites/default/files/store/content_1.jpeg',
        'https://avtonom.org/sites/default/files/store/fb_img_1658957390897_1.jpg',
        'https://avtonom.org/sites/default/files/store/tur_anarch.jpeg',
        'https://avtonom.org/sites/default/files/store/20220728_192615.jpg',
        'https://avtonom.org/sites/default/files/store/photo_2022-04-13_16-32-03_0.jpg',
        'https://avtonom.org/sites/default/files/store/20220728_125837.jpg',
        'https://avtonom.org/sites/default/files/store/20220726_221640.jpg',
        'https://avtonom.org/sites/default/files/store/wrench.jpg',
        'https://avtonom.org/sites/default/files/store/20220723_164055.jpg',
        'https://avtonom.org/sites/default/files/store/20220722_200142.jpg',
        'https://avtonom.org/sites/default/files/store/20220722_174617.jpg',
        'https://avtonom.org/sites/default/files/store/20220719_191947.jpg',
        'https://avtonom.org/sites/default/files/store/photo_2022-02-19_13-03-31_0.jpg',
        'https://avtonom.org/sites/default/files/store/20220718_172606_0.jpg',
    ]

    assert results[0][0][:100] == '<p><a href="https://avtonom.org/news/mosgorsud-ostavil-dmitriya-ivanova-v-sizo-0">Мосгорсуд оставил '
    assert results[0][0][-99:] == 'w">ФСИН-письмо</a>: Иванов Дмитрий Александрович, 1999 г.р., СИЗО «Матросская тишина».</strong></p>'
    assert results[1][0][:100] == '<p><a href="https://avtonom.org/news/petr-ryabov-rossiyskie-anarhisty-v-2000-2015-godah">Петр Рябов:'
    assert results[1][0][-99:] == 'торик, философ, публицист, исследователь истории и философии анархизма и экзистенциализма.</em></p>'
    assert results[-2][0][:100] == '<p><a href="https://avtonom.org/freenews/achk-bristol-rip-teylor-yarost-nashe-oruzhie">АЧК-Бристоль:'
    assert results[-2][0][-99:] == 'witter-timeline-link" href="https://bristolabc.org/riptaylor/" rel="nofollow">Источник</a></em></p>'
    assert results[-1][0][:100] == '<p><a href="https://avtonom.org/news/poka-vse-ne-budut-svobodny-mezhdunarodnaya-nedelya-solidarnosti'
    assert results[-1][0][-98:] == 'darnosti-s-zaklyuchennymi-anarhistami-2022-23-30-avgust/" rel="nofollow"><em>Источник</em></a></p>'


def test_process_new_items(get_raw_mock_avtonom, db):
    results = []
    def _processor(content, media):
        results.append((content, media))
        return 'T'
    rss.process_new_items(db, 'http://py.text/feed', _processor)
    assert len(results) == 20

    results = []
    def _processor(content, media):
        results.append((content, media))
        return 'T'
    rss.process_new_items(db, 'http://py.text/feed', _processor)
    assert results == []


def test_process_new_items_exception(get_raw_mock_avtonom, db):
    results = []
    def _processor(content, media):
        if media == 'https://avtonom.org/sites/default/files/store/wrench.jpg':
            raise RuntimeError()

        results.append((content, media))
        return 'T'
    rss.process_new_items(db, 'http://py.text/feed', _processor)
    assert len(results) == 19

    results = []
    def _processor(content, media):
        results.append((content, media))
        return 'T'
    rss.process_new_items(db, 'http://py.text/feed', _processor)
    assert len(results) == 1

    assert [media for content, media in results] == [
        'https://avtonom.org/sites/default/files/store/wrench.jpg',
    ]

    results = []
    def _processor(content, media):
        results.append((content, media))
        return 'T'
    rss.process_new_items(db, 'http://py.text/feed', _processor)
    assert results == []


def test_opennet(get_raw_mock_opennet, db):
    results = []
    def _processor(content, media):
        results.append((content, media))
        return 'T'
    rss.process_new_items(db, 'https://avtonom.org-pytest/rss_full', _processor)

    assert [media for content, media in results] == [None, ] * 15

    assert results[0][0][:100] == '<p><a href="https://www.opennet.ru/opennews/art.shtml?num=57600">Доступна операционная система Chrom'
    assert results[0][0][-100:] == 'ми  также формируются неофициальные сборки для обычных компьютеров с процессорами x86, x86_64 и ARM.'
    assert results[1][0][:100] == '<p><a href="https://www.opennet.ru/opennews/art.shtml?num=57598">Выпуск браузера Pale Moon 31.2</a><'
    assert results[1][0][-100:] == 's и Linux (x86 и x86_64). Код проекта распространяется под лицензией MPLv2 (Mozilla Public License).'
    assert results[-2][0][:100] == '<p><a href="https://www.opennet.ru/opennews/art.shtml?num=57617">ReactOS удалось запустить на систем'
    assert results[-2][0][-100:] == 'opennet.ru/opennews/art.shtml?num=57617">https://www.opennet.ru/opennews/art.shtml?num=57617</a></b>'
    assert results[-1][0][:100] == '<p><a href="https://www.opennet.ru/opennews/art.shtml?num=57614">В NPM планируют использовать Sigsto'
    assert results[-1][0][-100:] == 'opennet.ru/opennews/art.shtml?num=57614">https://www.opennet.ru/opennews/art.shtml?num=57614</a></b>'
